package com.jdbernard.net

import org.eclipse.jetty.rewrite.handler.RewriteHandler
import org.eclipse.jetty.rewrite.handler.RewriteRegexRule
import org.eclipse.jetty.servlet.DefaultServlet
import org.eclipse.jetty.servlet.ServletContextHandler
import org.eclipse.jetty.servlet.ServletHolder
import org.eclipse.jetty.server.Server
import groovy.servlet.GroovyServlet
import com.jdbernard.util.LightOptionParser

public class GroovyDirectoryServer {

    public static final String VERSION = "2.0"

    public static void main(String[] args) {
        def port = 9000

        def cli = [
            v: [longName: 'version'],
            h: [longName: 'help'],
            p: [longName: 'port', arguments: 1],
            P: [longName: 'map-path', arguments: 2],
            r: [longName: 'rewrite', arguments: 2],
            g: [longName: 'groovy-servlet', arguments: 1] ]

        def opts = LightOptionParser.parseOptions(cli, args)

        if (opts.v) { println "GroovyDirectoryServer v$VERSION"; System.exit(1) }
        if (opts.h) { println getUsage(); System.exit(1) }

        def options = [
            port: opts.p ? opts.p[0] as int : 9000,
            mappedPaths: opts.P ?: [['/', '.']],
            rewriteRules: opts.r ?: [],
            groovyPatterns: opts.g ?: ["*.groovy"] ]

        runJetty(options).join()
    }

    public static Server runJetty(options) {
        def server = new Server(options.port)

        def rewriteHandler = null
        def context = new ServletContextHandler(ServletContextHandler.SESSIONS)
        context.contextPath = '/'

        // Groovy Scripts
        options.groovyPatterns.eachWithIndex { pattern, idx ->
            def servletHolder = new ServletHolder("groovy-$idx", GroovyServlet)
            context.addServlet(servletHolder, pattern)
            println "Using GroovyServlet for urls matching '$pattern'." }

        // Mapped paths.
        def rootPair = options.mappedPaths.find { pair -> pair[0] == '/' }
        if (rootPair) {
            options.mappedPaths -= [rootPair]
            context.resourceBase = rootPair[1] }
        else {
            println "No root path specified: defaulting to the current directory."
            context.resourceBase = '.' }

        options.mappedPaths.eachWithIndex { pair, idx ->
            def servletHolder = new ServletHolder("static-$idx", DefaultServlet)
            servletHolder.setInitParameter('resourceBase', pair[1])
            servletHolder.setInitParameter('dirAllowed', 'true')
            servletHolder.setInitParameter('pathInfoOnly', 'true')
            context.addServlet(servletHolder, pair[0])
            println "Serving '${pair[1]}' from base url '${pair[0]}'." }

        def rootServletHolder = new ServletHolder('default', DefaultServlet)
        rootServletHolder.setInitParameter('dirAllowed', 'true')
        context.addServlet(rootServletHolder, '/')

        // Rewrite rules
        if (options.rewriteRules) {
            rewriteHandler = new RewriteHandler()
            rewriteHandler.setHandler(context) }

        options.rewriteRules.each { pair ->
            def rule = new RewriteRegexRule()
            rule.regex = pair[0]
            rule.replacement = pair[1]
            rewriteHandler.addRule(rule)
            println "Rewriting '${pair[0]}' to '${pair[1]}'." }

        server.handler = rewriteHandler ?: context
        server.start()

        println "Jetty started on port ${options.port}."

        return server
    }

    public static String getUsage() {
        return """\
GroovyDirectoryServer v$VERSION
usage: GroovyDirectoryServer [options]

Options:

  -h,--help         Print this usage information.

  -v, --version     Print versioning information.

  -p, --port <port> Listen on the given port. Defaults to 9000.

  -P <url-path> <dir-path>, --map-path <url-path> <dir-path>

    Map the contents of the filesystem at <dir-path> to the url prefix at
    <url-path>.

  -r <url-pattern> <replacement>, --rewrite <url-pattern> <replacement>

    Rewrite URLs matching <url-pattern> to <replacement>.

  -g <url=pattern>, --groovy-servlet

    Invoke the GroovyServet for any url matching <url-pattern>
""";
    }
}
